(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (factory((global.angularjsTest = {})));
}(this, (function (exports) { 'use strict';

  var compose = function compose() {
    for (var _len = arguments.length, functions = Array(_len), _key = 0; _key < _len; _key++) {
      functions[_key] = arguments[_key];
    }
    return functions.reduce(function (f, g) {
      return function () {
        return f(g.apply(undefined, arguments));
      };
    });
  };
  var convertKebabCaseToCamelCase = function convertKebabCaseToCamelCase(string) {
    return string.replace(/(-\w)/g, function (m) {
      return m[1].toUpperCase();
    });
  };

  function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }
  function mockComponent(kebabCaseName) {
    var name = convertKebabCaseToCamelCase(kebabCaseName);
    function mock($provide) {
      $provide.decorator(name + 'Directive', function ($delegate) {
        var component = $delegate[0];
        component.controller = function controller() {
          mock._controller = this;
        };
        return $delegate;
      });
    }
    angular.mock.module(mock);
    return withMethods(mock);
  }
  function withMethods(mock) {
    return compose(withExists, withProps, withProp, withSimulate)(mock);
  }
  function withExists(mock) {
    mock.exists = function () {
      return !!mock._controller;
    };
    return mock;
  }
  function withProps(mock) {
    mock.props = function () {
      var _mock$_controller = mock._controller,
          controller = _mock$_controller.controller,
          props = _objectWithoutProperties(_mock$_controller, ['controller']);
      return props;
    };
    return mock;
  }
  function withProp(mock) {
    mock.prop = function (key) {
      return mock._controller[key];
    };
    return mock;
  }
  function withSimulate(mock) {
    mock.simulate = function (event, data) {
      var callbackName = 'on' + event[0].toUpperCase() + event.slice(1);
      mock._controller[callbackName]({ $event: data });
      return mock;
    };
    return mock;
  }

  var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };
  function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
  function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }
  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
  var TestElementWrapper = function TestElementWrapper(angularElement) {
    var _this = this;
    _classCallCheck(this, TestElementWrapper);
    this.length = angularElement.length;
    this.html = function () {
      return angularElement.html();
    };
    this.text = function () {
      return angularElement.text();
    };
    this.hasClass = function (name) {
      return angularElement.hasClass(name);
    };
    this.exists = function () {
      return angularElement && angularElement.length > 0;
    };
    this.find = function (selector) {
      var foundElement = angularElement[0].querySelectorAll(selector);
      return elementToTestElementWrapper(foundElement);
    };
    this.map = function (fn) {
      return [].concat(_toConsumableArray(angularElement)).map(elementToTestElementWrapper).map(fn);
    };
    this.props = function () {
      return [].concat(_toConsumableArray(angularElement[0].attributes)).reduce(function (props, _ref) {
        var name = _ref.name,
            value = _ref.value;
        return _extends({}, props, _defineProperty({}, name, value));
      }, {});
    };
    this.prop = function (key) {
      return angularElement.attr(key);
    };
    this.simulate = function (event, data) {
      if (event === 'change') {
        if (!data || !data.target) {
          throw new Error('Simulating change expects an event object ({ target: { value } })');
        }
        angularElement.val(data.target.value).triggerHandler('input');
        return _this;
      }
      angularElement.triggerHandler(event);
      return _this;
    };
    this.setProps = function (props) {
      var $rootScope = void 0;
      angular.mock.inject(function ($injector) {
        $rootScope = $injector.get('$rootScope');
      });
      $rootScope.$ctrl = _extends({}, $rootScope.$ctrl, props);
      $rootScope.$digest();
      return _this;
    };
  };
  function elementToTestElementWrapper(element) {
    return new TestElementWrapper(angular.element(element));
  }

  function mount(template, props) {
    var $rootScope = void 0;
    var element = void 0;
    angular.mock.inject(function ($compile, $injector) {
      $rootScope = $injector.get('$rootScope');
      $rootScope.$ctrl = props;
      element = $compile(template)($rootScope);
    });
    $rootScope.$digest();
    return new TestElementWrapper(element);
  }

  exports.mockComponent = mockComponent;
  exports.mount = mount;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
